FILE_DURATION = 99999
"""
Valid int duration for specific shortcodes and enhance elements
that need to calculate the real duration in the last moment
when the resource is available because the duration will be that
resource duration.
"""
SEGMENT_DURATION = 999999
"""
Valid int duration for specific enhancement elements that need to
calculate the real duration in the last moment when the whole
segment video is available because the duration will be that 
segment duration.
"""